package com.lina.branch.branch

import android.app.Activity
import android.util.Log
import androidx.annotation.NonNull
import io.branch.referral.Branch
import io.branch.referral.Branch.BranchReferralInitListener
import io.branch.referral.BranchError
import io.branch.referral.util.BRANCH_STANDARD_EVENT
import io.branch.referral.util.BranchEvent
import io.branch.referral.util.CurrencyType

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONObject

/** BranchPlugin */
class BranchPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private lateinit var channel: MethodChannel
    private var activity: Activity? = null
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "branch")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val event = BRANCH_STANDARD_EVENT.valueOf(call.method)
        if (BRANCH_STANDARD_EVENT.values().toList().contains(event)) {
            report(call.method, data = call.arguments as Map<String?, Any?>)
        }

    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
        try {
            Branch.sessionBuilder(activity).withCallback(branchReferralInitListener).init()
        } catch (e: Exception) {
        }
    }
    private val branchReferralInitListener =
        BranchReferralInitListener { linkProperties, error ->
            if (error != null) {
                Log.w("Branch 初始化", error.message)
            } else {
                Log.w("Branch 初始化", linkProperties.toString())
            }
        }
    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
        activity = null
    }

    //注册，付费金额
    private fun report(eventName: String, data: Map<String?, Any?>) {
        if (activity == null) {
            return
        }
        val event = BranchEvent(eventName)
        //添加自动一参数
        for (key in data.keys) {
            event.addCustomDataProperty(key, data[key].toString())
        }
        ///上报支付需要添加 金额 和 币种
        if (BRANCH_STANDARD_EVENT.PURCHASE.name == eventName) {
            event.setRevenue(data["revenue"] as Double)
            event.setCurrency(CurrencyType.valueOf(data["currency"].toString()))
        }

        event.logEvent(activity)
    }
}
